<?php

namespace FM\Service;

use FM\Factory\ApontamentoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FileServerService;

class ApontamentoService
{

    protected $ApontamentoFactory;
    protected $FileServerService;
    protected $environment;
    protected $newCode;

    public function __construct(EntityManager $entityManager, ApontamentoFactory $ApontamentoFactory, FileServerService $FileServerService, Environment $environment, NewCode $newCode)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->ApontamentoFactory           = $ApontamentoFactory;
        $this->FileServerService            = $FileServerService;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
    }

    public function setNewRegister($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->environment->getNrOrg();
            $cdOperador = $this->environment->getCdOperador();

            $todayrRegister = $this->ApontamentoFactory->getTodayRegister($cdOperador);
            $qtRegisters = $todayrRegister[0]['QTDAP'];
            $NRAPONTAMENTO = null;

            if ($qtRegisters == 0) {
                $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
                $this->ApontamentoFactory->insertNewAppointment(array(
                    "NRAPONTAMENTO" => $NRAPONTAMENTO,
                    "CDOPERADOR" => $this->environment->getCdOperador(),
                    "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
                    "IDFALTA"       => 'N',
                    "IDAPURADO"     => 'N',
                    "NRORG"         => $NRORG
                ));
            }

            if ($NRAPONTAMENTO == null) {
                $nr = $this->ApontamentoFactory->getNRAP($cdOperador);
                $NRAPONTAMENTO = $nr[0]['NRAPONTAMENTO'];
            }

            $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

            $lastEntry = $this->ApontamentoFactory->lastEntry($cdOperador);
            if (!empty($lastEntry)) {
                $dateEntry = $lastEntry[0]['DTHRENTRADA'];
            } else {
                $dateEntry = null;
            }

            $this->ApontamentoFactory->insertNewRegister(array(
                "NRREGISTROHORAS" => $NRREGISTROHORAS,
                "NRAPONTAMENTO"  => $NRAPONTAMENTO,
                "DTHRENTRADA"    => $parametros['DTHRENTRADA'],
                "IDTIPORESGISTRO" => $parametros['IDTIPORESGISTRO'],
                "HRTEMPOREGISTRO" => ($parametros['IDTIPORESGISTRO'] === 'S') ? null : 0,
                "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
                "IDALMOCO"       => $parametros['IDALMOCO'],
                "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
                "NRORG"          => $NRORG,
                "DATEENTRY"      => $dateEntry
            ));

            if (isset($parametros['IMAGENS'])) {
                $this->inserirAnexo($parametros['IMAGENS'], $NRREGISTROHORAS);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getUltimoRegistro()
    {
        return $this->ApontamentoFactory->getUltimoRegistro($this->environment->getCdOperador());
    }

    public function inserirAnexo($arrayAnexos, $nrRegistroHoras)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->environment->getNrOrg();
            $cdOperador = $this->environment->getCdOperador();

            $imagens = $this->FileServerService->uploadFiles($arrayAnexos, 'anexosApontamento');

            foreach ($imagens as $imagem) {
                $this->ApontamentoFactory->inserirAnexos(array(
                    "NRANEXOSAPONTAMENTO"   => $this->newCode->getCodeByTableName("FCM_ANEXOSAPONTAMENTO", $NRORG, 1, 10, true),
                    "NRREGISTROHORAS"       => $nrRegistroHoras,
                    "DSCAMINHOANEXO"        => $imagem['path'],
                    "NRORG"                 => $NRORG,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $NRORG,
                    "DSTYPE"                => $imagem['type'],
                    "NRSIZE"                => $imagem['size'],
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function optionLunch()
    {
        $cdOperador = $this->environment->getCdOperador();

        $qtLunch = $this->ApontamentoFactory->optionLunch($cdOperador);
        $types = $this->ApontamentoFactory->getUltimoRegistro($cdOperador);

        $retorno = array();
        array_push($retorno, (!$qtLunch) ? 0 : $qtLunch[0], $types[0]);
        return $retorno;
    }

    public function getApontamentos()
    {
        $cdOperador = $this->environment->getCdOperador();
        return $this->ApontamentoFactory->getApontamentos($cdOperador);
    }

    public function setAbsence($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->environment->getNrOrg();
            $NRAPONTAMENTO = $this->newCode->getCodeByTableName("FCM_APONTAMENTO", $NRORG, 1, 10, true);
            $paramsAppointment = array(
                "NRAPONTAMENTO" => $NRAPONTAMENTO,
                "CDOPERADOR" => $this->environment->getCdOperador(),
                "DTAPONTAMENTO" => $parametros['DTAPONTAMENTO'],
                "IDFALTA"       => 'S',
                "IDAPURADO"     => 'N',
                "NRORG"         => $NRORG
            );
            $this->ApontamentoFactory->insertNewAppointment($paramsAppointment);

            $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

            $parametrosNewRegister = array(
                "NRREGISTROHORAS" => $NRREGISTROHORAS,
                "NRAPONTAMENTO"  => $NRAPONTAMENTO,
                "DTHRENTRADA"    => null,
                "IDTIPORESGISTRO" => null,
                "HRTEMPOREGISTRO" => null,
                "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
                "IDALMOCO"       => 'N',
                "DTHRSAIDA"      => null,
                "NRORG"          => $NRORG,
                "DATEENTRY"      => null,
            );

            $this->ApontamentoFactory->insertNewRegister($parametrosNewRegister);

            $this->inserirAnexo($parametros['IMAGENS'], $NRREGISTROHORAS);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function setLastAppointment($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->environment->getNrOrg();
            $NRUSUARIOLOGADO = $this->environment->getCdOperador();
            $NRREGISTROHORAS = $this->newCode->getCodeByTableName("FCM_REGISTROHORAS", $NRORG, 1, 10, true);

            $lastEntry = $this->ApontamentoFactory->lastEntry($NRUSUARIOLOGADO);

            if (!empty($lastEntry)) {
                $dateEntry = $lastEntry[0]['DTHRENTRADA'];
            } else {
                $dateEntry = null;
            }

            $this->ApontamentoFactory->insertNewRegister(array(
                "NRREGISTROHORAS" => $NRREGISTROHORAS,
                "NRAPONTAMENTO"  => $parametros['NRAPONTAMENTO'],
                "DTHRENTRADA"    => null,
                "IDTIPORESGISTRO" => 'S',
                "HRTEMPOREGISTRO" => null,
                "DSOBSREGISTRO"  => $parametros['DSOBSREGISTRO'],
                "IDALMOCO"       => 'N',
                "DTHRSAIDA"      => $parametros['DTHRSAIDA'],
                "NRORG"          => $NRORG,
                "DATEENTRY"      => $dateEntry,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
