<?php

namespace FM\Service;

use FM\Factory\Fm00002VistoriasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FuncionariosHierarqService;
use FM\Service\FindService;
use FM\Service\FileServerService;

class Fm00002VistoriasService
{

    protected $connection;
    protected $VistoriasFactory;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $FileServerService;
    protected $Environment;
    protected $newCode;
    protected $fileServerUrl;

    public function __construct(EntityManager $entityManager, Fm00002VistoriasFactory $VistoriasFactory, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, Environment $Environment, NewCode $newCode, FileServerService $FileServerService, $fileServerUrl)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->VistoriasFactory             = $VistoriasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
        $this->FileServerService            = $FileServerService;
        $this->fileServerUrl                = $fileServerUrl;
    }

    public function getVistoriasListadas($parametros)
    {
        $filtro = '';
        $cdOperador = $this->Environment->getCdOperador();
        $retorno = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdOperador);


        if ($parametros['TIPO'] == "2") {
            if ($parametros['NRGERENCIAMENTOCHK'] != '' && $parametros['NRGERENCIAMENTOCHK'] != null) {
                $filtro = $filtro . " AND GCHK.NRGERENCIAMENTOCHK = " . $parametros['NRGERENCIAMENTOCHK'];
            }
        } elseif ($parametros['TIPO'] == "3") {
            if (isset($parametros['NRSEQCONTRATO'])) {
                $contratos = $this->FindService->formatarArray($parametros['NRSEQCONTRATO']);
                $filtro = $filtro . " AND GCHK.NRSEQCONTRATO IN ( " . $contratos . ")";
            }

            if (isset($parametros['CDCLIENTE'])) {
                $clientes = $this->FindService->formatarArray($parametros['CDCLIENTE']);
                $filtro = $filtro . " AND LC.CDCLIENTE IN ( " . $clientes . ")";
            }

            if (isset($parametros['NRLOCALCLIENTE'])) {
                $local = $this->FindService->formatarArray($parametros['NRLOCALCLIENTE']);
                $filtro = $filtro . " AND GCHK.NRLOCALCLIENTE IN ( " . $local . ")";
            }

            if (isset($parametros['CDOPERADOR'])) {
                $operadores = $this->FindService->formatarArray($parametros['CDOPERADOR']);
                $filtro = $filtro . " AND GCHK.CDOPERADOR IN ( " . $operadores . ")";
            }

            if (isset($parametros['IDSITUACAOGERCHK'])) {
                $situacao = $this->FindService->formatarArray($parametros['IDSITUACAOGERCHK']);
                $filtro = $filtro . " AND GCHK.IDSITUACAOGERCHK IN ( " . $situacao . ")";
            }

            if (isset($parametros['DTHORAGEND'])) {
                $dataInicial = $parametros['DTHORAGEND'][0];
                $dataFinal = array_key_exists("1", $parametros['DTHORAGEND']) ? $parametros['DTHORAGEND'][1] : null;

                if ($dataFinal != null) {
                    $filtro = $filtro . " AND GCHK.DTHORAAGEND BETWEEN '" . $dataInicial . "' AND '" . $dataFinal . "'";
                } else {
                    $data = $dataInicial;
                    $filtro = $filtro . " AND GCHK.DTHORAAGEND BETWEEN '" . $data . "' AND TO_DATE('" . $data . "')+30";
                }
            }
        }

        return $this->VistoriasFactory->getVistoriasListadas($retorno, $filtro);
    }

    public function getFotosVistorias($parametros)
    {
        return $this->VistoriasFactory->getFotosVistorias(array(
            "NRGERENCIAMENTOCHK"    => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }

    public function excluirFotoVistorias($parametros)
    {
        return $this->VistoriasFactory->excluirFotoVistorias(array(
            "NRFOTOTAREFAGERCHK"    => $parametros['NRFOTOTAREFAGERCHK'],
        ));
    }

    public function getAtividades($parametros)
    {
        return $this->VistoriasFactory->getAtividades(array(
            "NRGERENCIAMENTOCHK"    => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }

    public function anexarImagensVistoria($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $NRORG = $this->Environment->getNrOrg();
            $cdOperador = $this->Environment->getCdOperador();

            $codTarefa = $this->VistoriasFactory->getCodTarefaGerChk(array(
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
            ));

            $imagens = $this->FileServerService->uploadFiles($parametros['IMAGENS'], 'anexosTarefa');
        
            foreach ($imagens as $img) {
                $this->VistoriasFactory->anexarImagemVistorias(array(
                    "NRFOTOTAREFAGERCHK"    => $this->newCode->getCodeByTableName("FCM_FOTOTAREFAGERCHK", $NRORG, 1, 10, true),
                    "NRTAREFAGERCHK"        => $codTarefa[0]['NRTAREFAGERCHK'],
                    "DSFOTOTAREFAGERCHK"    => $this->fileServerUrl . '/files/' . $img['path'],
                    "NRORG"                 => $NRORG,
                    "CDOPERINCLUSAO"        => $cdOperador,
                    "NRORGINCLUSAO"         => $NRORG,
                    "DSTYPE"                => $img['type'],
                    "NRSIZE"                => $img['size']
                ));
            }

            $this->VistoriasFactory->updateStatusVistoria(array(
                "IDSITUACAOGERCHK"   => '2',
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getCriterios($parametros)
    {
        return $this->VistoriasFactory->getCriterios(array(
            "NRATIVIDADECHK"        => $parametros['NRATIVIDADECHK'],
            "NRGERENCIAMENTOCHK"    => $parametros['NRGERENCIAMENTOCHK'],
        ));
    }

    public function salvarCriterios($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            $this->VistoriasFactory->updateCriterios(array(
                "NRATIVIDADECHK"     => $parametros['NRATIVIDADECHK'],
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "IDMARCADO"          => "N",
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ), "");

            if (!empty($parametros['CRITERIOS'])) {
                $criterios = $this->FindService->formatarArray($parametros['CRITERIOS']);

                $paramQuery = " AND NMCRITERIOTARGERCHK IN(" . $criterios . ")";

                $this->VistoriasFactory->updateCriterios(array(
                    "NRATIVIDADECHK"     => $parametros['NRATIVIDADECHK'],
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "IDMARCADO"          => 'S',
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG,
                ), $paramQuery);
            }

            $this->VistoriasFactory->updateStatusVistoria(array(
                "IDSITUACAOGERCHK"   => '2',
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ));

            $NMGERENCIAMENTOCHK = $this->VistoriasFactory->getNomeVistoria(array(
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK']
            ));

            $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true);
            $parametroHistoricoVistoria = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "NMGERENCIAMENTOCHK" => $NMGERENCIAMENTOCHK[0]['NMGERENCIAMENTOCHK'],
                    "DSGERENCIAMENTOCHK" => "Vistoria em andamento.",
                    "IDTIPOHISTORICO"    => 3,
                    "NRORG"              => $NRORG,
                    "CDOPERINCLUSAO"     => $cdOperador,
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG
                );
    
                $this->VistoriasFactory->atualizaHistoricoVistoria($parametroHistoricoVistoria);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function finalizarVistoria($parametros)
    {
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        $cdOperador = $this->Environment->getCdOperador();

        try {
            foreach ($parametros['ATIVIDADES'] as $atividade) {
                $this->VistoriasFactory->updateObservacao(array(
                    "NRATIVIDADECHK"     => $atividade,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "DSOBSTAREFAGERCHK"  => $parametros['DSOBSTAREFAGERCHK'],
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG,
                ));
            }
            
            $this->VistoriasFactory->updateStatusVistoria(array(
                "IDSITUACAOGERCHK"   => '3',
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                "CDOPERULTATU"       => $cdOperador,
                "NRORGULTATU"        => $NRORG,
            ));

            $NMGERENCIAMENTOCHK = $this->VistoriasFactory->getNomeVistoria(array(
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK']
            ));

            $tarefa = $this->VistoriasFactory->getTarefaVinculada(array(
                "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
            ));

            if (!$parametros['POSSUINAOCONFORM']) {
                $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true);
                $parametroHistoricoVistoria = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "NMGERENCIAMENTOCHK" => $NMGERENCIAMENTOCHK[0]['NMGERENCIAMENTOCHK'],
                    "DSGERENCIAMENTOCHK" => "Vistoria finalizada sem não conformidades.",
                    "IDTIPOHISTORICO"    => 4,
                    "NRORG"              => $NRORG,
                    "CDOPERINCLUSAO"     => $cdOperador,
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG
                );
    
                $this->VistoriasFactory->atualizaHistoricoVistoria($parametroHistoricoVistoria);

                $NRCHKAGENDHISTORICO = $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true);
		        $parametroHistoricoTarefa = array(
                    "NRCHKAGENDHISTORICO" => $NRCHKAGENDHISTORICO,
                    "NRCHECKLISTAGEND"    => $tarefa[0]['NRCHECKLISTAGEND'],
                    "NMCHECKLISTAGEND"    => $tarefa[0]['NMCHECKLISTAGEND'],
                    "DSCHKAGENDHISTORICO" => "Tarefa avaliada e finalizada.",
                    "IDTIPOHISTORICO"     => 4,
                    "NRORG"               => $NRORG,
                    "CDOPERINCLUSAO"      => $cdOperador,
                    "NRORGINCLUSAO"       => $NRORG,
                    "CDOPERULTATU"        => $cdOperador,
                    "NRORGULTATU"         => $NRORG
                );

                $this->VistoriasFactory->atualizaHistoricoTarefa($parametroHistoricoTarefa);
            } else {
                $NRGERCHKHISTORICO = $this->newCode->getCodeByTableName("FCM_GERCHKHISTORICO", $NRORG, 1, 10, true);
                $parametroHistoricoVistoria = array(
                    "NRGERCHKHISTORICO"  => $NRGERCHKHISTORICO,
                    "NRGERENCIAMENTOCHK" => $parametros['NRGERENCIAMENTOCHK'],
                    "NMGERENCIAMENTOCHK" => $NMGERENCIAMENTOCHK[0]['NMGERENCIAMENTOCHK'],
                    "DSGERENCIAMENTOCHK" => "Vistoria finalizada contendo não conformidades.",
                    "IDTIPOHISTORICO"    => 6,
                    "NRORG"              => $NRORG,
                    "CDOPERINCLUSAO"     => $cdOperador,
                    "NRORGINCLUSAO"      => $NRORG,
                    "CDOPERULTATU"       => $cdOperador,
                    "NRORGULTATU"        => $NRORG
                );
    
                $this->VistoriasFactory->atualizaHistoricoVistoria($parametroHistoricoVistoria);

                $NRCHKAGENDHISTORICO = $this->newCode->getCodeByTableName("FCM_CHKAGENDHISTORICO", $NRORG, 1, 10, true);
		        $parametroHistoricoTarefa = array(
                    "NRCHKAGENDHISTORICO" => $NRCHKAGENDHISTORICO,
                    "NRCHECKLISTAGEND"    => $tarefa[0]['NRCHECKLISTAGEND'],
                    "NMCHECKLISTAGEND"    => $tarefa[0]['NMCHECKLISTAGEND'],
                    "DSCHKAGENDHISTORICO" => "Tarefa avaliada e finalizada contendo não conformidades.",
                    "IDTIPOHISTORICO"     => 6,
                    "NRORG"               => $NRORG,
                    "CDOPERINCLUSAO"      => $cdOperador,
                    "NRORGINCLUSAO"       => $NRORG,
                    "CDOPERULTATU"        => $cdOperador,
                    "NRORGULTATU"         => $NRORG
                );

                $this->VistoriasFactory->atualizaHistoricoTarefa($parametroHistoricoTarefa);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}
